SystemOrganization addCategory: #'Cutie-Helvetia-Automaton'!

QQTestCase subclass: #CUAutomatonTest
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cutie-Helvetia-Automaton'!

!CUAutomatonTest methodsFor: 'running' stamp: 'lr 4/27/2010 11:58'!
setUp
	super setUp.
	repository := CUAutomatonRepository new! !

!CUAutomatonTest methodsFor: 'testing' stamp: 'lr 4/27/2010 12:00'!
testExample
	self assert: (repository example accept: 'car').
	self assert: (repository example accept: 'cdr').
	self assert: (repository example accept: 'cadr').
	self assert: (repository example accept: 'cdar').	
	self assert: (repository example accept: 'caadr').
	self assert: (repository example accept: 'cdaar').
	self assert: (repository example accept: 'caddr').
	self assert: (repository example accept: 'cddar')! !

Object subclass: #CUAutomaton
	instanceVariableNames: 'states initial current'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cutie-Helvetia-Automaton'!

!CUAutomaton methodsFor: 'public' stamp: 'lr 11/11/2009 17:35'!
accept: aString
	current := states at: initial.
	aString do: [ :char | 
		(current accept: char in: self)
			ifFalse: [ ^ false ] ].
	^ true! !

!CUAutomaton methodsFor: 'creational' stamp: 'lr 11/11/2009 13:08'!
initialState: aString
	initial := aString! !

!CUAutomaton methodsFor: 'initialization' stamp: 'lr 11/11/2009 11:54'!
initialize
	states := Dictionary new! !

!CUAutomaton methodsFor: 'creational' stamp: 'lr 11/11/2009 17:36'!
newState: aString
	^ states at: aString put: CUAutomatonState new! !

!CUAutomaton methodsFor: 'initialization' stamp: 'lr 11/11/2009 11:57'!
setCurrent: aState
	current := aState! !

!CUAutomaton methodsFor: 'accessing' stamp: 'lr 11/11/2009 11:40'!
states
	^ states! !

Object subclass: #CUAutomatonRepository
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cutie-Helvetia-Automaton'!

!CUAutomatonRepository class methodsFor: 'helvetia' stamp: 'lr 5/4/2010 09:14'!
highlightAutomaton
	<highlight>
	
	^ CUAutomatonHighlighter new! !

!CUAutomatonRepository class methodsFor: 'private' stamp: 'lr 11/11/2009 14:00'!
hightlightSmalltalk80	
	^ nil! !

!CUAutomatonRepository class methodsFor: 'helvetia' stamp: 'lr 5/4/2010 09:25'!
parseAutomaton
	<parse>
	
	^ [ :context |
		| stream result |
		result := CUAutomatonCompiler new
			parse: context stream.
		result isPetitFailure
			ifTrue: [ CHParserError signal: result reason at: result position ].
		RBMethodNode
			selector: result first value asSymbol
			body: (RBSequenceNode statements: (Array 
				with: (RBReturnNode
					value: (QQObjectNode
						value: result last)))) ]! !

!CUAutomatonRepository methodsFor: 'accessing' stamp: 'lr 5/4/2010 09:28'!
example
	init
	init :	c -> more
	more :	a -> more
			d -> more
			r -> end
	end :! !

Object subclass: #CUAutomatonState
	instanceVariableNames: 'transitions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cutie-Helvetia-Automaton'!

!CUAutomatonState methodsFor: 'public' stamp: 'lr 11/11/2009 17:35'!
accept: aCharacter in: anAutomaton
	| name |
	name := transitions at: aCharacter ifAbsent: [ ^ false ].
	anAutomaton setCurrent: (anAutomaton states
		at: name
		ifAbsent: [ self error: 'Unknown state ' , name ]).
	^ true! !

!CUAutomatonState methodsFor: 'initialization' stamp: 'lr 11/11/2009 17:34'!
initialize
	transitions := Dictionary new! !

!CUAutomatonState methodsFor: 'public' stamp: 'lr 11/11/2009 11:57'!
on: aCharacter do: aSymbol
	transitions at: aCharacter put: aSymbol ! !

PPCompositeParser subclass: #CUAutomatonGrammar
	instanceVariableNames: 'identifier character automaton initial state transition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cutie-Helvetia-Automaton'!

CUAutomatonGrammar subclass: #CUAutomatonCompiler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cutie-Helvetia-Automaton'!

!CUAutomatonCompiler methodsFor: 'productions' stamp: 'lr 11/11/2009 13:52'!
automaton
	^ super automaton ==> [ :nodes |
		| result |
		result := CUAutomaton new.
		result initialState: nodes first value.
		nodes second do: [ :each |
			| newState |
			newState := result newState: each key.
			each value do: [ :assoc |
				newState on: assoc key do: assoc value ] ].
		result ]! !

!CUAutomatonCompiler methodsFor: 'productions' stamp: 'lr 11/11/2009 13:19'!
state
	^ super state ==> [ :nodes | nodes first value -> nodes last ]! !

!CUAutomatonCompiler methodsFor: 'productions' stamp: 'lr 11/11/2009 13:20'!
transition
	^ super transition ==> [ :nodes | nodes first value first -> nodes last value ]! !

!CUAutomatonGrammar methodsFor: 'productions' stamp: 'lr 11/11/2009 13:51'!
automaton
	^ initial , state plus! !

!CUAutomatonGrammar methodsFor: 'tokens' stamp: 'lr 4/26/2010 23:31'!
character
	^ #letter asParser token trim! !

!CUAutomatonGrammar methodsFor: 'tokens' stamp: 'lr 4/26/2010 23:31'!
identifier
	^ (#letter asParser , #word asParser plus) token trim! !

!CUAutomatonGrammar methodsFor: 'productions' stamp: 'lr 11/11/2009 13:07'!
initial
	^ identifier! !

!CUAutomatonGrammar methodsFor: 'accessing' stamp: 'lr 5/4/2010 09:23'!
start
	^ identifier , automaton end! !

!CUAutomatonGrammar methodsFor: 'productions' stamp: 'lr 4/26/2010 23:30'!
state
	^ identifier , $: asParser token trim , transition star! !

!CUAutomatonGrammar methodsFor: 'productions' stamp: 'lr 4/26/2010 23:30'!
transition
	^ character , '->' asParser token trim , identifier! !

CUAutomatonGrammar subclass: #CUAutomatonHighlighter
	instanceVariableNames: 'context'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cutie-Helvetia-Automaton'!

CUAutomatonHighlighter class
	instanceVariableNames: 'text'!

CUAutomatonHighlighter class
	instanceVariableNames: 'text'!

!CUAutomatonHighlighter class methodsFor: 'as yet unclassified' stamp: 'lr 5/4/2010 09:21'!
ignoredNames
	^ super ignoredNames , #('context')! !

!CUAutomatonHighlighter methodsFor: 'as yet unclassified' stamp: 'lr 5/4/2010 09:20'!
acceptDsl: aContext
	context := aContext.
	self parse: aContext text! !

!CUAutomatonHighlighter methodsFor: 'tokens' stamp: 'lr 5/4/2010 09:28'!
character
	^ super character ==> [ :token | self highlight: token with: (Color r: 0 g: 0.5 b: 0) ]! !

!CUAutomatonHighlighter methodsFor: 'private' stamp: 'lr 5/4/2010 09:19'!
highlight: aToken with: aColor
	context scopeFrom: aToken start to: aToken stop with: nil visit: aColor! !

!CUAutomatonHighlighter methodsFor: 'tokens' stamp: 'lr 5/4/2010 09:27'!
identifier
	^ super identifier ==> [ :token | self highlight: token with: (Color r: 0 g: 0 b: 0.5) ]! !
